/*!
* @Author: richen
* @Date: 2024-04-15 05:09:35
* @License: BSD (3-Clause)
* @Copyright (c) - <richenlin(at)gmail.com>
* @HomePage: https://koatty.org/
*/
import { AppEvent } from 'koatty_core';
import { Config } from 'koatty_config';
import { EventHookFunc } from 'koatty_core';
import { Helper } from 'koatty_lib';
import { IAspect } from 'koatty_container';
import { Koatty } from 'koatty_core';
import { KoattyContext } from 'koatty_core';
import { Logger as Logger_2 } from 'koatty_logger';
import { Middleware as Middleware_2 } from 'koa';
import { Next } from 'koa';
import { Value } from 'koatty_config';

/**
 * @description: bind App event hook func
 * example:
 * export function TestDecorator(): ClassDecorator {
 *  return (target: Function) => {
 *   BindEventHook(AppEvent.appBoot, (app: Koatty) => {
 *      // todo
 *      return Promise.resolve();
 *   }, target)
 *  }
 * }
 * @param {AppEvent} eventName
 * @param {EventHookFunc} eventFunc
 * @param {any} target
 * @return {*}
 */
export declare function BindEventHook(eventName: AppEvent, eventFunc: EventHookFunc, target: any): void;

/**
 * Bootstrap application
 *
 * @export
 * @param {Function} [bootFunc]
 * @returns {ClassDecorator}
 */
export declare function Bootstrap(bootFunc?: Function): ClassDecorator;

/**
 * Indicates that an decorated class is a "component".
 *
 * @export
 * @param {string} [identifier] component name
 * @returns {ClassDecorator}
 */
export declare function Component(identifier?: string): ClassDecorator;

/**
 * Define project scan path
 *
 * @export
 * @param {(string | string[])} [scanPath]
 * @returns {ClassDecorator}
 */
export declare function ComponentScan(scanPath?: string | string[]): ClassDecorator;

export { Config }

/**
 * Define project configuration scan path
 *
 * @export
 * @param {(string | string[])} [scanPath]
 * @returns {ClassDecorator}
 */
export declare function ConfigurationScan(scanPath?: string | string[]): ClassDecorator;

/**
 * Indicates that an decorated class is a "controller".
 *
 * @export
 * @param {string} [path] controller router path
 * @returns {ClassDecorator}
 */
export declare function Controller(path?: string): ClassDecorator;

/**
 * Actively perform dependency injection
 * Parse the decorator, return the instantiated app.
 * @export  ExecBootStrap
 * @param {Function} [bootFunc] callback function
 * @returns
 */
export declare function ExecBootStrap(bootFunc?: Function): (target: any) => Promise<Koatty>;

export { Helper }

/**
 * Interface for Controller
 */
export declare interface IController {
    readonly app: Koatty;
    readonly ctx: KoattyContext;
}

/**
 * Interface for Middleware
 */
export declare interface IMiddleware {
    run: (options: any, app: Koatty) => (ctx: KoattyContext, next: Next) => Promise<any>;
}

/**
 * check is implements Aspect Interface
 * @param cls
 * @returns
 */
export declare function implementsAspectInterface(cls: any): cls is IAspect;

/**
 * check is implements Controller Interface
 * @param cls
 * @returns
 */
export declare function implementsControllerInterface(cls: any): cls is IController;

/**
 * check is implements Middleware Interface
 * @param cls
 * @returns
 */
export declare function implementsMiddlewareInterface(cls: any): cls is IMiddleware;

/**
 * check is implements Plugin Interface
 * @param cls
 * @returns
 */
export declare function implementsPluginInterface(cls: any): cls is IPlugin;

/**
 * check is implements Service Interface
 * @param cls
 * @returns
 */
export declare function implementsServiceInterface(cls: any): cls is IService;

/**
 * Interface for Plugin
 */
export declare interface IPlugin {
    run: (options: any, app: Koatty) => Promise<any>;
}

/**
 * Interface for Service
 */
export declare interface IService {
    readonly app: Koatty;
}

/**
 * Interface for Middleware
 */
export declare interface KoattyMiddleware extends Middleware_2 {
}

export declare const Logger: Logger_2;

/**
 * Indicates that an decorated class is a "middleware".
 *
 * @export
 * @param {string} [identifier] class name
 * @returns {ClassDecorator}
 */
export declare function Middleware(identifier?: string): ClassDecorator;

/**
 * Indicates that an decorated class is a "plugin".
 *
 * @export
 * @param {string} [identifier] class name
 * @returns {ClassDecorator}
 */
export declare function Plugin(identifier?: string): ClassDecorator;

/**
 * Indicates that an decorated class is a "service".
 *
 * @export
 * @param {string} [identifier] class name
 * @returns {ClassDecorator}
 */
export declare function Service(identifier?: string): ClassDecorator;

export { Value }


export * from "koatty_container";
export * from "koatty_core";
export * from "koatty_exception";
export * from "koatty_router";
export * from "koatty_serve";

export { }
