/*!
* @Author: richen
* @Date: 2024-04-15 05:09:24
* @License: BSD (3-Clause)
* @Copyright (c) - <richenlin(at)gmail.com>
* @HomePage: https://koatty.org/
*/
'use strict';

var koatty_router = require('koatty_router');
var koatty_container = require('koatty_container');
var koatty_core = require('koatty_core');
var koatty_serve = require('koatty_serve');
var path = require('path');
var koatty_loader = require('koatty_loader');
var koatty_exception = require('koatty_exception');
var koatty_config = require('koatty_config');
var koatty_trace = require('koatty_trace');
var koatty_lib = require('koatty_lib');
var koatty_logger = require('koatty_logger');
require('reflect-metadata');

function _interopNamespaceDefault(e) {
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var path__namespace = /*#__PURE__*/_interopNamespaceDefault(path);

function TraceHandler(app) {
    const timeout = (app.config('http_timeout') || 10) * 1000;
    const encoding = app.config('encoding') || 'utf-8';
    const openTrace = app.config("open_trace") || false;
    const asyncHooks = app.config("async_hooks") || false;
    const options = {
        RequestIdHeaderName: app.config('trace_header') || 'X-Request-Id',
        RequestIdName: app.config('trace_id') || "requestId",
        IdFactory: undefined,
        Timeout: timeout,
        Encoding: encoding,
        OpenTrace: openTrace,
        AsyncHooks: asyncHooks,
    };
    app.use(koatty_trace.Trace(options, app));
}

/*
 * @Description: framework helper
 * @Usage:
 * @Author: richen
 * @Date: 2023-12-09 21:56:32
 * @LastEditTime: 2024-01-16 01:20:17
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
/**
 * Check class file
 * name should be always the same as class name
 * class must be unique
 *
 * @export
 * @param {string} fileName
 * @param {string} xpath
 * @param {*} target
 * @param {Set<unknown>} [exSet]
 * @returns {*}
 */
function checkClass(fileName, xpath, target, exSet) {
    if (koatty_lib.Helper.isClass(target) && target.name != fileName) { // export default class name{}
        throw Error(`The file(${xpath}) name should be always the same as class name.`);
    }
    if (target["__esModule"]) {
        if (target.name === undefined) { // export class name{}
            const keys = Object.keys(target);
            if (keys[0] != fileName && koatty_lib.Helper.isClass(target[keys[0]])) {
                throw Error(`The file(${xpath}) name should be always the same as class name.`);
            }
        }
        else if (target.name != fileName) { // export default class {}
            throw Error(`The file(${xpath}) name should be always the same as class name.`);
        }
    }
    if (!exSet) {
        return;
    }
    if (exSet.has(fileName)) {
        throw new Error(`A same class already exists. at \`${xpath}\`.`);
    }
    exSet.add(fileName);
    return;
}

/*
 * @Description: framework logger
 * @Usage:
 * @Author: richen
 * @Date: 2023-12-09 21:56:32
 * @LastEditTime: 2023-12-09 23:02:27
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
// Logger
const Logger = koatty_logger.DefaultLogger;
/**
 * SetLogger
 *
 * @export
 * @param {{
 *     logLevel?: LogLevelType;
 *     logFilePath?: string;
 *     sensFields?: string[];
 * }} config
 */
function SetLogger(app, config) {
    if (config.logLevel) {
        koatty_logger.DefaultLogger.setLevel(config.logLevel);
    }
    if (config.logFilePath && !app.silent) {
        koatty_lib.Helper.define(app, "logsPath", config.logFilePath);
        process.env.LOGS_PATH = config.logFilePath;
        koatty_logger.DefaultLogger.setLogFilePath(config.logFilePath);
    }
    if (config.sensFields) {
        koatty_logger.DefaultLogger.setSensFields(config.sensFields);
    }
}

/*
 * @Description: component interface
 * @Usage:
 * @Author: richen
 * @Date: 2023-12-09 21:56:32
 * @LastEditTime: 2024-01-19 08:36:27
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
// tslint:disable-next-line: no-import-side-effect
/**
 * Indicates that an decorated class is a "component".
 *
 * @export
 * @param {string} [identifier] component name
 * @returns {ClassDecorator}
 */
function Component(identifier) {
    return (target) => {
        identifier = identifier || koatty_container.IOCContainer.getIdentifier(target);
        koatty_container.IOCContainer.saveClass("COMPONENT", target, identifier);
    };
}
/**
 * Indicates that an decorated class is a "controller".
 *
 * @export
 * @param {string} [path] controller router path
 * @returns {ClassDecorator}
 */
function Controller(path = "") {
    return (target) => {
        const identifier = koatty_container.IOCContainer.getIdentifier(target);
        koatty_container.IOCContainer.saveClass("CONTROLLER", target, identifier);
        koatty_container.IOCContainer.savePropertyData(koatty_router.CONTROLLER_ROUTER, path, target, identifier);
    };
}
/**
 * Indicates that an decorated class is a "middleware".
 *
 * @export
 * @param {string} [identifier] class name
 * @returns {ClassDecorator}
 */
function Middleware(identifier) {
    return (target) => {
        identifier = identifier || koatty_container.IOCContainer.getIdentifier(target);
        koatty_container.IOCContainer.saveClass("MIDDLEWARE", target, identifier);
    };
}
/**
 * Indicates that an decorated class is a "service".
 *
 * @export
 * @param {string} [identifier] class name
 * @returns {ClassDecorator}
 */
function Service(identifier) {
    return (target) => {
        identifier = identifier || koatty_container.IOCContainer.getIdentifier(target);
        koatty_container.IOCContainer.saveClass("SERVICE", target, identifier);
    };
}
/**
 * Indicates that an decorated class is a "plugin".
 *
 * @export
 * @param {string} [identifier] class name
 * @returns {ClassDecorator}
 */
function Plugin(identifier) {
    return (target) => {
        identifier = identifier || koatty_container.IOCContainer.getIdentifier(target);
        // 
        if (!identifier.endsWith("Plugin")) {
            throw Error("Plugin class name must be 'Plugin' suffix.");
        }
        koatty_container.IOCContainer.saveClass("COMPONENT", target, `${identifier}`);
    };
}
/**
 * check is implements Middleware Interface
 * @param cls
 * @returns
 */
function implementsMiddlewareInterface(cls) {
    return 'run' in cls && koatty_lib.Helper.isFunction(cls.run);
}
/**
 * check is implements Controller Interface
 * @param cls
 * @returns
 */
function implementsControllerInterface(cls) {
    return 'app' in cls && 'ctx' in cls;
}
/**
 * check is implements Service Interface
 * @param cls
 * @returns
 */
function implementsServiceInterface(cls) {
    return 'app' in cls;
}
/**
 * check is implements Plugin Interface
 * @param cls
 * @returns
 */
function implementsPluginInterface(cls) {
    return 'run' in cls && koatty_lib.Helper.isFunction(cls.run);
}
/**
 * check is implements Aspect Interface
 * @param cls
 * @returns
 */
function implementsAspectInterface(cls) {
    return 'app' in cls && 'run' in cls && koatty_lib.Helper.isFunction(cls.run);
}

/*
 * @Description: framework constants
 * @Usage:
 * @Author: richen
 * @Date: 2023-12-09 21:56:32
 * @LastEditTime: 2023-12-09 23:00:13
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
const COMPONENT_SCAN = 'COMPONENT_SCAN';
const CONFIGURATION_SCAN = 'CONFIGURATION_SCAN';
// tslint:disable: no-irregular-whitespace
const LOGO = `

┬┌─┌─┐┌─┐┌┬┐┌┬┐┬ ┬
├┴┐│ │├─┤ │  │ └┬┘
┴ ┴└─┘┴ ┴ ┴  ┴  ┴ 
-------------------------------------------
https://github.com/koatty
`;

/*
 * @Description: framework loader
 * @Usage:
 * @Author: richen
 * @Date: 2023-12-09 22:55:49
 * @LastEditTime: 2024-01-24 11:54:29
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
/**
 *
 */
class Loader {
    /**
     * initialize env
     *
     * @static
     * @param {Koatty} app
     * @memberof Loader
     */
    static initialize(app) {
        const env = (process.execArgv ?? []).join(",");
        if (env.indexOf('ts-node') > -1 || env.indexOf('--debug') > -1) {
            app.appDebug = true;
        }
        // app.env
        app.env = process.env.KOATTY_ENV || process.env.NODE_ENV;
        if ((env.indexOf('--production') > -1) || ((app.env ?? '').indexOf('pro') > -1)) {
            app.appDebug = false;
        }
        if (app.appDebug) {
            app.env = 'development';
            process.env.NODE_ENV = 'development';
            process.env.APP_DEBUG = 'true';
            Logger.setLevel("debug");
        }
        else {
            app.env = 'production';
            process.env.NODE_ENV = 'production';
            Logger.setLevel("info");
        }
        // define path
        const rootPath = app.rootPath || process.cwd();
        const appPath = app.appPath || path__namespace.resolve(rootPath, env.indexOf('ts-node') > -1 ? 'src' : 'dist');
        const koattyPath = path__namespace.resolve(__dirname, '..');
        koatty_lib.Helper.define(app, 'rootPath', rootPath);
        koatty_lib.Helper.define(app, 'appPath', appPath);
        koatty_lib.Helper.define(app, 'koattyPath', koattyPath);
        // 
        if (koatty_lib.Helper.isEmpty(app.name)) {
            const pkg = koatty_lib.Helper.safeRequire(`${path__namespace.dirname(appPath)}/package.json`);
            if (pkg.name) {
                app.name = pkg.name;
                app.version = app.version || pkg.version;
            }
        }
        process.env.ROOT_PATH = rootPath;
        process.env.APP_PATH = appPath;
        process.env.KOATTY_PATH = koattyPath;
        // Compatible with old version, will be deprecated
        koatty_lib.Helper.define(app, 'prevent', koatty_exception.prevent);
        koatty_lib.Helper.define(app, 'thinkPath', koattyPath);
        process.env.THINK_PATH = koattyPath;
    }
    /**
     * Get component metadata
     *
     * @static
     * @param {Koatty} app
     * @param {*} target
     * @returns {*}  {any[]}
     * @memberof Loader
     */
    static GetComponentMetas(app, target) {
        let componentMetas = [];
        const componentMeta = koatty_container.IOCContainer.getClassMetadata(koatty_container.TAGGED_CLS, COMPONENT_SCAN, target);
        if (componentMeta) {
            if (koatty_lib.Helper.isArray(componentMeta)) {
                componentMetas = componentMeta;
            }
            else {
                componentMetas.push(componentMeta);
            }
        }
        if (componentMetas.length < 1) {
            componentMetas = [app.appPath];
        }
        return componentMetas;
    }
    /**
     * Load all bean, excepted config/*、App.ts
     *
     * @static
     * @param {Koatty} app
     * @param {*} target
     * @memberof Loader
     */
    static CheckAllComponents(app, target) {
        // component metadata
        const componentMetas = Loader.GetComponentMetas(app, target);
        // configuration metadata
        const configurationMetas = Loader.GetConfigurationMetas(app, target);
        const exSet = new Set();
        koatty_loader.Load(componentMetas, '', (fileName, xpath, xTarget) => {
            checkClass(fileName, xpath, xTarget, exSet);
        }, ['**/**.js', '**/**.ts', '!**/**.d.ts'], [...configurationMetas, `${target.name || '.no'}.ts`]);
        exSet.clear();
    }
    /**
     * Get configuration metadata
     *
     * @static
     * @param {Koatty} app
     * @param {*} target
     * @returns {*}  {any[]}
     * @memberof Loader
     */
    static GetConfigurationMetas(app, target) {
        const confMeta = koatty_container.IOCContainer.getClassMetadata(koatty_container.TAGGED_CLS, CONFIGURATION_SCAN, target);
        let configurationMetas = [];
        if (confMeta) {
            if (koatty_lib.Helper.isArray(confMeta)) {
                configurationMetas = confMeta;
            }
            else {
                configurationMetas.push(confMeta);
            }
        }
        return configurationMetas;
    }
    /**
     * Set Logger level
     *
     * @static
     * @param {Koatty} app
     * @memberof Loader
     */
    static SetLogger(app) {
        const data = app.getMetaData('_configs') || [];
        const configs = data[0] || {};
        //Logger
        if (configs.config) {
            const opt = configs.config;
            let logLevel = "debug", logFilePath = "", sensFields = [];
            if (app.env === "production") {
                logLevel = "info";
            }
            if (opt.logs_level) {
                logLevel = (opt.logs_level).toLowerCase();
            }
            if (opt.logs_path) {
                logFilePath = opt.logs_path;
            }
            if (opt.sens_fields) {
                sensFields = opt.sens_fields;
            }
            SetLogger(app, { logLevel, logFilePath, sensFields });
        }
    }
    /**
     * Load app event hook funcs
     *
     * @static
     * @param {Koatty} app
     * @param {*} target
     * @memberof Loader
     */
    static LoadAppEventHooks(app, target) {
        const eventFuncs = new Map();
        for (const event of koatty_core.AppEventArr) {
            let funcs;
            switch (event) {
                case "appBoot" /* AppEvent.appBoot */:
                    funcs = koatty_container.IOCContainer.getClassMetadata(koatty_container.TAGGED_CLS, "appBoot" /* AppEvent.appBoot */, target);
                    if (koatty_lib.Helper.isArray(funcs)) {
                        eventFuncs.set("appBoot" /* AppEvent.appBoot */, funcs);
                    }
                    break;
                case "appReady" /* AppEvent.appReady */:
                    funcs = koatty_container.IOCContainer.getClassMetadata(koatty_container.TAGGED_CLS, "appReady" /* AppEvent.appReady */, target);
                    if (koatty_lib.Helper.isArray(funcs)) {
                        eventFuncs.set("appReady" /* AppEvent.appReady */, funcs);
                    }
                    break;
                case "appStart" /* AppEvent.appStart */:
                    funcs = koatty_container.IOCContainer.getClassMetadata(koatty_container.TAGGED_CLS, "appStart" /* AppEvent.appStart */, target);
                    if (koatty_lib.Helper.isArray(funcs)) {
                        eventFuncs.set("appStart" /* AppEvent.appStart */, funcs);
                    }
                    break;
                case "appStop" /* AppEvent.appStop */:
                    funcs = koatty_container.IOCContainer.getClassMetadata(koatty_container.TAGGED_CLS, "appStop" /* AppEvent.appStop */, target);
                    if (koatty_lib.Helper.isArray(funcs)) {
                        eventFuncs.set("appStop" /* AppEvent.appStop */, funcs);
                    }
                    break;
            }
        }
        // loop event emit
        for (const [event, funcs] of eventFuncs) {
            for (const func of funcs) {
                app.once(event, () => func(app));
            }
        }
    }
    /**
     * Load configuration
     *
     * @static
     * @param {Koatty} app
     * @param {string[]} [loadPath]
     * @memberof Loader
     */
    static LoadConfigs(app, loadPath) {
        const frameConfig = {};
        // Logger.Debug(`Load configuration path: ${app.thinkPath}/config`);
        koatty_loader.Load(["./config"], app.koattyPath, function (name, path, exp) {
            frameConfig[name] = exp;
        });
        if (koatty_lib.Helper.isArray(loadPath)) {
            loadPath = loadPath.length > 0 ? loadPath : ["./config"];
        }
        let appConfig = koatty_config.LoadConfigs(loadPath, app.appPath);
        appConfig = koatty_lib.Helper.extend(frameConfig, appConfig, true);
        app.setMetaData("_configs", appConfig);
    }
    /**
     * Load middlewares
     * [async]
     * @static
     * @param {*} app
     * @param {(string | string[])} [loadPath]
     * @memberof Loader
     */
    static async LoadMiddlewares(app, loadPath) {
        // Error handling middleware
        await TraceHandler(app);
        let middlewareConf = app.config(undefined, "middleware");
        if (koatty_lib.Helper.isEmpty(middlewareConf)) {
            middlewareConf = { config: {}, list: [] };
        }
        //Mount default middleware
        // Load(loadPath || ["./middleware"], app.koattyPath);
        //Mount application middleware
        // const middleware: any = {};
        const appMiddleware = koatty_container.IOCContainer.listClass("MIDDLEWARE") ?? [];
        appMiddleware.forEach((item) => {
            item.id = (item.id ?? "").replace("MIDDLEWARE:", "");
            if (item.id && koatty_lib.Helper.isClass(item.target)) {
                koatty_container.IOCContainer.reg(item.id, item.target, { scope: "Prototype", type: "MIDDLEWARE", args: [] });
                const ctl = koatty_container.IOCContainer.getInsByClass(item.target);
                if (!implementsMiddlewareInterface(ctl)) {
                    throw Error(`The middleware ${item.id} must implements interface 'IMiddleware'.`);
                }
            }
        });
        const middlewareConfList = middlewareConf.list || [];
        //de-duplication
        const appMList = new Set([]);
        middlewareConfList.forEach((item) => {
            appMList.add(item);
        });
        //Automatically call middleware
        const middlewareConfig = middlewareConf.config || {};
        for (const key of appMList) {
            const handle = koatty_container.IOCContainer.get(key, "MIDDLEWARE");
            if (!handle) {
                throw Error(`Middleware ${key} load error.`);
            }
            if (!koatty_lib.Helper.isFunction(handle.run)) {
                throw Error(`The middleware ${key} must implements interface 'IMiddleware'.`);
            }
            if (middlewareConfig[key] === false) {
                Logger.Warn(`The middleware ${key} has been loaded but not executed.`);
                continue;
            }
            Logger.Debug(`Load middleware: ${key}`);
            const result = await handle.run(middlewareConfig[key] || {}, app);
            if (koatty_lib.Helper.isFunction(result)) {
                if (result.length < 3) {
                    app.use(result);
                }
                else {
                    app.useExp(result);
                }
            }
        }
    }
    /**
     * Load controllers
     *
     * @static
     * @param {*} app
     * @memberof Loader
     */
    static LoadControllers(app) {
        const controllerList = koatty_container.IOCContainer.listClass("CONTROLLER");
        const controllers = [];
        controllerList.forEach((item) => {
            item.id = (item.id ?? "").replace("CONTROLLER:", "");
            if (item.id && koatty_lib.Helper.isClass(item.target)) {
                Logger.Debug(`Load controller: ${item.id}`);
                // registering to IOC
                koatty_container.IOCContainer.reg(item.id, item.target, { scope: "Prototype", type: "CONTROLLER", args: [] });
                const ctl = koatty_container.IOCContainer.getInsByClass(item.target);
                if (!implementsControllerInterface(ctl)) {
                    throw Error(`The controller ${item.id} must implements interface 'IController'.`);
                }
                controllers.push(item.id);
            }
        });
        return controllers;
    }
    /**
     * Load services
     *
     * @static
     * @param {*} app
     * @memberof Loader
     */
    static LoadServices(app) {
        const serviceList = koatty_container.IOCContainer.listClass("SERVICE");
        serviceList.forEach((item) => {
            item.id = (item.id ?? "").replace("SERVICE:", "");
            if (item.id && koatty_lib.Helper.isClass(item.target)) {
                Logger.Debug(`Load service: ${item.id}`);
                // registering to IOC
                koatty_container.IOCContainer.reg(item.id, item.target, { scope: "Singleton", type: "SERVICE", args: [] });
                const ctl = koatty_container.IOCContainer.getInsByClass(item.target);
                if (!implementsServiceInterface(ctl)) {
                    throw Error(`The service ${item.id} must implements interface 'IService'.`);
                }
            }
        });
    }
    /**
     * Load components
     *
     * @static
     * @param {*} app
     * @memberof Loader
     */
    static async LoadComponents(app) {
        const componentList = koatty_container.IOCContainer.listClass("COMPONENT");
        const pluginList = [];
        componentList.forEach(async (item) => {
            item.id = (item.id ?? "").replace("COMPONENT:", "");
            if (koatty_lib.Helper.isClass(item.target)) {
                // registering to IOC
                koatty_container.IOCContainer.reg(item.id, item.target, { scope: "Singleton", type: "COMPONENT", args: [] });
                if (item.id && (item.id).endsWith("Plugin")) {
                    const ctl = koatty_container.IOCContainer.getInsByClass(item.target);
                    if (!implementsPluginInterface(ctl)) {
                        throw Error(`The plugin ${item.id} must implements interface 'IPlugin'.`);
                    }
                    pluginList.push(item.id);
                }
                if (item.id && (item.id).endsWith("Aspect")) {
                    const ctl = koatty_container.IOCContainer.getInsByClass(item.target);
                    if (!implementsAspectInterface(ctl)) {
                        throw Error(`The aspect ${item.id} must implements interface 'IAspect'.`);
                    }
                }
            }
        });
        // load plugin config
        let pluginsConf = app.config(undefined, "plugin");
        if (koatty_lib.Helper.isEmpty(pluginsConf)) {
            pluginsConf = { config: {}, list: [] };
        }
        const pluginConfList = pluginsConf.list ?? [];
        // load plugin list
        for (const key of pluginConfList) {
            const handle = koatty_container.IOCContainer.get(key, "COMPONENT");
            if (!handle) {
                throw Error(`Plugin ${key} load error.`);
            }
            if (!koatty_lib.Helper.isFunction(handle.run)) {
                throw Error(`Plugin ${key} must implements interface 'IPlugin'.`);
            }
            if (pluginsConf.config[key] === false) {
                Logger.Warn(`Plugin ${key} already loaded but not effective.`);
                continue;
            }
            // sync exec 
            await handle.run(pluginsConf.config[key] ?? {}, app);
        }
    }
}

var version = "3.11.4-2";
var engines = {
	node: ">12.0.0"
};

/*
 * @Description: framework runtime checker
 * @Usage:
 * @Author: richen
 * @Date: 2023-12-09 21:56:32
 * @LastEditTime: 2023-12-09 23:01:22
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
const KOATTY_VERSION = version;
const ENGINES_VERSION = engines.node.slice(1) || '12.0.0';
/**
 * check node version
 * @return {void} []
 */
function checkRuntime() {
    let nodeEngines = ENGINES_VERSION;
    nodeEngines = nodeEngines.slice(0, nodeEngines.lastIndexOf('.'));
    let nodeVersion = process.version;
    if (nodeVersion[0] === 'v') {
        nodeVersion = nodeVersion.slice(1);
    }
    nodeVersion = nodeVersion.slice(0, nodeVersion.lastIndexOf('.'));
    if (koatty_lib.Helper.toNumber(nodeEngines) > koatty_lib.Helper.toNumber(nodeVersion)) {
        Logger.Error(`Koatty need node version > ${nodeEngines}, current version is ${nodeVersion}, please upgrade it.`);
        process.exit(-1);
    }
}
/**
 * unittest running environment detection
 * only support jest
 * @returns {boolean}
 */
const checkUTRuntime = () => {
    let isUTRuntime = false;
    // UT运行环境判断，暂时先只判断jest
    const argv = JSON.stringify(process.argv[1]);
    if (argv.indexOf('jest') > -1) {
        isUTRuntime = true;
    }
    return isUTRuntime;
};

/*
 * @Description: framework bootstrap
 * @Usage:
 * @Author: richen
 * @Date: 2021-12-09 21:56:32
 * @LastEditTime: 2024-01-16 01:06:26
 * @License: BSD (3-Clause)
 * @Copyright (c): <richenlin(at)gmail.com>
 */
/**
 * execute bootstrap
 *
 * @param {*} target
 * @param {Function} bootFunc
 * @param {boolean} [isInitiative=false] Whether to actively execute app instantiation,
 * mainly for unittest scenarios, you need to actively obtain app instances
 * @returns {Promise<void>}
 */
const executeBootstrap = async function (target, bootFunc, isInitiative = false) {
    // checked runtime
    checkRuntime();
    // unittest running environment
    const isUTRuntime = checkUTRuntime();
    if (!isInitiative && isUTRuntime) {
        return;
    }
    const app = Reflect.construct(target, []);
    // unittest does not print startup logs
    if (isUTRuntime) {
        app.silent = true;
        Logger.enable(false);
    }
    try {
        !app.silent && Logger.Log("Koatty", LOGO);
        if (!(app instanceof koatty_core.Koatty)) {
            throw new Error(`class ${target.name} does not inherit from Koatty`);
        }
        // Initialize env
        Loader.initialize(app);
        // exec bootFunc
        if (koatty_lib.Helper.isFunction(bootFunc)) {
            Logger.Log('Koatty', '', 'Execute bootFunc ...');
            await bootFunc(app);
        }
        // Set IOCContainer.app
        koatty_container.IOCContainer.setApp(app);
        Logger.Log('Koatty', '', 'ComponentScan ...');
        // Check all bean
        Loader.CheckAllComponents(app, target);
        // Load configuration
        Logger.Log('Koatty', '', 'Load Configurations ...');
        // configuration metadata
        const configurationMetas = Loader.GetConfigurationMetas(app, target);
        Loader.LoadConfigs(app, configurationMetas);
        // Load App event hooks
        Loader.LoadAppEventHooks(app, target);
        Logger.Log('Koatty', '', 'Emit App Boot ...');
        await asyncEvent(app, "appBoot" /* AppEvent.appBoot */);
        // Create Server
        // app.server = newServe(app);
        koatty_lib.Helper.define(app, "server", koatty_serve.NewServe(app));
        // Create router
        // app.router = newRouter(app);
        koatty_lib.Helper.define(app, "router", koatty_router.NewRouter(app));
        // Load Components
        Logger.Log('Koatty', '', 'Load Components ...');
        await Loader.LoadComponents(app);
        // Load Middleware
        Logger.Log('Koatty', '', 'Load Middlewares ...');
        await Loader.LoadMiddlewares(app);
        // Load Services
        Logger.Log('Koatty', '', 'Load Services ...');
        Loader.LoadServices(app);
        // Load Controllers
        Logger.Log('Koatty', '', 'Load Controllers ...');
        const controllers = Loader.LoadControllers(app);
        // Emit app ready event
        Logger.Log('Koatty', '', 'Emit App Ready ...');
        await asyncEvent(app, "appReady" /* AppEvent.appReady */);
        // Load Routers
        Logger.Log('Koatty', '', 'Load Routers ...');
        app.router.LoadRouter(app, controllers);
        if (!isUTRuntime) {
            // Start Server
            app.listen(listenCallback);
        }
        return app;
    }
    catch (err) {
        Logger.Error(err);
        process.exit();
    }
};
/**
 * Listening callback function
 *
 * @param {Koatty} app
 * @returns {*}
 */
const listenCallback = (app) => {
    const options = app.server.options;
    Logger.Log('Koatty', '', '====================================');
    Logger.Log("Koatty", "", `Nodejs Version: ${process.version}`);
    Logger.Log("Koatty", "", `Koatty Version: v${KOATTY_VERSION}`);
    Logger.Log("Koatty", "", `App Environment: ${app.env}`);
    Logger.Log('Koatty', '', `Server Protocol: ${(options.protocol).toUpperCase()}`);
    Logger.Log("Koatty", "", `Server running at ${options.protocol === "http2" ? "https" : options.protocol}://${options.hostname || '127.0.0.1'}:${options.port}/`);
    Logger.Log("Koatty", "", "====================================");
    // binding event "appStop"
    Logger.Log('Koatty', '', 'Bind App Stop event ...');
    koatty_serve.BindProcessEvent(app, 'appStop');
    // tslint:disable-next-line: no-unused-expression
    app.appDebug && Logger.Warn(`Running in debug mode.`);
    // Set Logger
    Loader.SetLogger(app);
};
/**
 * Execute event as async
 *
 * @param {Koatty} event
 * @param {string} eventName
 */
const asyncEvent = async function (event, eventName) {
    const ls = event.listeners(eventName);
    // eslint-disable-next-line no-restricted-syntax
    for await (const func of ls) {
        if (koatty_lib.Helper.isFunction(func)) {
            func();
        }
    }
    return event.removeAllListeners(eventName);
};
/**
 * Bootstrap application
 *
 * @export
 * @param {Function} [bootFunc]
 * @returns {ClassDecorator}
 */
function Bootstrap(bootFunc) {
    return function (target) {
        if (!(target.prototype instanceof koatty_core.Koatty)) {
            throw new Error(`class does not inherit from Koatty`);
        }
        executeBootstrap(target, bootFunc);
    };
}
/**
 * Actively perform dependency injection
 * Parse the decorator, return the instantiated app.
 * @export  ExecBootStrap
 * @param {Function} [bootFunc] callback function
 * @returns
 */
function ExecBootStrap(bootFunc) {
    return async (target) => {
        if (!(target.prototype instanceof koatty_core.Koatty)) {
            throw new Error(`class ${target.name} does not inherit from Koatty`);
        }
        return await executeBootstrap(target, bootFunc, true);
    };
}
/**
 * Define project scan path
 *
 * @export
 * @param {(string | string[])} [scanPath]
 * @returns {ClassDecorator}
 */
function ComponentScan(scanPath) {
    return (target) => {
        if (!(target.prototype instanceof koatty_core.Koatty)) {
            throw new Error(`class does not inherit from Koatty`);
        }
        scanPath = scanPath ?? '';
        koatty_container.IOCContainer.saveClassMetadata(koatty_container.TAGGED_CLS, COMPONENT_SCAN, scanPath, target);
    };
}
/**
 * Define project configuration scan path
 *
 * @export
 * @param {(string | string[])} [scanPath]
 * @returns {ClassDecorator}
 */
function ConfigurationScan(scanPath) {
    return (target) => {
        if (!(target.prototype instanceof koatty_core.Koatty)) {
            throw new Error(`class does not inherit from Koatty`);
        }
        scanPath = scanPath ?? '';
        koatty_container.IOCContainer.saveClassMetadata(koatty_container.TAGGED_CLS, CONFIGURATION_SCAN, scanPath, target);
    };
}
/**
 * @description: bind App event hook func
 * example:
 * export function TestDecorator(): ClassDecorator {
 *  return (target: Function) => {
 *   BindEventHook(AppEvent.appBoot, (app: Koatty) => {
 *      // todo
 *      return Promise.resolve();
 *   }, target)
 *  }
 * }
 * @param {AppEvent} eventName
 * @param {EventHookFunc} eventFunc
 * @param {any} target
 * @return {*}
 */
function BindEventHook(eventName, eventFunc, target) {
    koatty_container.IOCContainer.attachClassMetadata(koatty_container.TAGGED_CLS, eventName, eventFunc, target);
}

Object.defineProperty(exports, 'Config', {
  enumerable: true,
  get: function () { return koatty_config.Config; }
});
Object.defineProperty(exports, 'Value', {
  enumerable: true,
  get: function () { return koatty_config.Value; }
});
Object.defineProperty(exports, 'Helper', {
  enumerable: true,
  get: function () { return koatty_lib.Helper; }
});
exports.BindEventHook = BindEventHook;
exports.Bootstrap = Bootstrap;
exports.Component = Component;
exports.ComponentScan = ComponentScan;
exports.ConfigurationScan = ConfigurationScan;
exports.Controller = Controller;
exports.ExecBootStrap = ExecBootStrap;
exports.Logger = Logger;
exports.Middleware = Middleware;
exports.Plugin = Plugin;
exports.Service = Service;
exports.implementsAspectInterface = implementsAspectInterface;
exports.implementsControllerInterface = implementsControllerInterface;
exports.implementsMiddlewareInterface = implementsMiddlewareInterface;
exports.implementsPluginInterface = implementsPluginInterface;
exports.implementsServiceInterface = implementsServiceInterface;
Object.keys(koatty_router).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return koatty_router[k]; }
  });
});
Object.keys(koatty_container).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return koatty_container[k]; }
  });
});
Object.keys(koatty_core).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return koatty_core[k]; }
  });
});
Object.keys(koatty_serve).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return koatty_serve[k]; }
  });
});
Object.keys(koatty_exception).forEach(function (k) {
  if (k !== 'default' && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return koatty_exception[k]; }
  });
});
